# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .query_response_items_records_item import QueryResponseItemsRecordsItem
from .query_summary import QuerySummary


class QueryResponseItems(UniversalBaseModel):
    """
    Response for line item queries
    """

    records: typing_extensions.Annotated[
        typing.Optional[typing.List[QueryResponseItemsRecordsItem]],
        FieldMetadata(alias="Records"),
        pydantic.Field(alias="Records"),
    ] = None
    summary: typing_extensions.Annotated[
        typing.Optional[QuerySummary], FieldMetadata(alias="Summary"), pydantic.Field(alias="Summary")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .method_element_settings_apple_pay_button_style import MethodElementSettingsApplePayButtonStyle
from .method_element_settings_apple_pay_button_type import MethodElementSettingsApplePayButtonType
from .method_element_settings_apple_pay_language import MethodElementSettingsApplePayLanguage


class MethodElementSettingsApplePay(UniversalBaseModel):
    button_style: typing_extensions.Annotated[
        typing.Optional[MethodElementSettingsApplePayButtonStyle],
        FieldMetadata(alias="buttonStyle"),
        pydantic.Field(
            alias="buttonStyle",
            description="The Apple Pay button style. See [Apple Pay Button Style](/developers/developer-guides/hosted-payment-page-apple-pay#param-applepay-button-style) for more information.",
        ),
    ] = None
    button_type: typing_extensions.Annotated[
        typing.Optional[MethodElementSettingsApplePayButtonType],
        FieldMetadata(alias="buttonType"),
        pydantic.Field(
            alias="buttonType",
            description="The text on Apple Pay button. See [Apple Pay Button Type](/developers/developer-guides/hosted-payment-page-apple-pay#param-applepay-button-type) for more information.",
        ),
    ] = None
    language: typing.Optional[MethodElementSettingsApplePayLanguage] = pydantic.Field(default=None)
    """
    The Apple Pay button locale. See [Apple Pay Button Language](/developers/developer-guides/hosted-payment-page-apple-pay#param-applepay-language) for more information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class StatisticsVendorQueryRecord(UniversalBaseModel):
    stat_x: typing_extensions.Annotated[
        str, FieldMetadata(alias="statX"), pydantic.Field(alias="statX", description="Statistical grouping identifier")
    ]
    active: int = pydantic.Field()
    """
    Number of active transactions
    """

    active_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="activeVolume"),
        pydantic.Field(alias="activeVolume", description="Volume of active transactions"),
    ]
    sent_to_approval: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="sentToApproval"),
        pydantic.Field(alias="sentToApproval", description="Number of transactions sent to approval"),
    ]
    sent_to_approval_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="sentToApprovalVolume"),
        pydantic.Field(alias="sentToApprovalVolume", description="Volume of transactions sent to approval"),
    ]
    to_approval: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="toApproval"),
        pydantic.Field(alias="toApproval", description="Number of transactions to approval"),
    ]
    to_approval_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="toApprovalVolume"),
        pydantic.Field(alias="toApprovalVolume", description="Volume of transactions to approval"),
    ]
    approved: int = pydantic.Field()
    """
    Number of approved transactions
    """

    approved_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="approvedVolume"),
        pydantic.Field(alias="approvedVolume", description="Volume of approved transactions"),
    ]
    disapproved: int = pydantic.Field()
    """
    Number of disapproved transactions
    """

    disapproved_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="disapprovedVolume"),
        pydantic.Field(alias="disapprovedVolume", description="Volume of disapproved transactions"),
    ]
    cancelled: int = pydantic.Field()
    """
    Number of cancelled transactions
    """

    cancelled_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="cancelledVolume"),
        pydantic.Field(alias="cancelledVolume", description="Volume of cancelled transactions"),
    ]
    in_transit: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="inTransit"),
        pydantic.Field(alias="inTransit", description="Number of transactions in transit"),
    ]
    in_transit_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="inTransitVolume"),
        pydantic.Field(alias="inTransitVolume", description="Volume of transactions in transit"),
    ]
    paid: int = pydantic.Field()
    """
    Number of paid transactions
    """

    paid_volume: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="paidVolume"),
        pydantic.Field(alias="paidVolume", description="Volume of paid transactions"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .enabled import Enabled
from .order import Order


class ContactElement(UniversalBaseModel):
    email_label: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="emailLabel"),
        pydantic.Field(alias="emailLabel", description="Custom content for email"),
    ] = None
    enabled: typing.Optional[Enabled] = None
    header: typing.Optional[str] = pydantic.Field(default=None)
    """
    Header text for section
    """

    order: typing.Optional[Order] = None
    payment_icons: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="paymentIcons"),
        pydantic.Field(
            alias="paymentIcons", description="Flag indicating if icons for accepted card brands will be shown"
        ),
    ] = None
    phone_label: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="phoneLabel"),
        pydantic.Field(alias="phoneLabel", description="Custom content for phone number"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .accounting_field import AccountingField
from .additional_data_string import AdditionalDataString
from .attachments import Attachments
from .comments import Comments
from .datenullable import Datenullable
from .invoice_number import InvoiceNumber
from .net_amountstring import NetAmountstring
from .terms import Terms


class BillPayOutDataRequest(UniversalBaseModel):
    bill_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="billId"),
        pydantic.Field(alias="billId", description="Bill ID in Payabli."),
    ] = None
    comments: typing.Optional[Comments] = pydantic.Field(default=None)
    """
    Any comments about bill. **For managed payouts, this field has a limit of 100 characters**.
    """

    due_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="dueDate"),
        pydantic.Field(alias="dueDate", description="Bill due date in format YYYY-MM-DD or MM/DD/YYYY."),
    ] = None
    invoice_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="invoiceDate"),
        pydantic.Field(alias="invoiceDate", description="Bill date in format YYYY-MM-DD or MM/DD/YYYY."),
    ] = None
    invoice_number: typing_extensions.Annotated[
        typing.Optional[InvoiceNumber],
        FieldMetadata(alias="invoiceNumber"),
        pydantic.Field(
            alias="invoiceNumber",
            description="Custom number identifying the bill. Must be unique in paypoint. **Required** for new bill and when `billId` isn't provided.",
        ),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[NetAmountstring],
        FieldMetadata(alias="netAmount"),
        pydantic.Field(alias="netAmount", description="Net Amount owed in bill. Required when adding a bill."),
    ] = None
    discount: typing.Optional[str] = pydantic.Field(default=None)
    """
    Bill discount amount.
    """

    terms: typing.Optional[Terms] = pydantic.Field(default=None)
    """
    Description of payment terms.
    """

    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[AccountingField],
        FieldMetadata(alias="accountingField1"),
        pydantic.Field(alias="accountingField1"),
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[AccountingField],
        FieldMetadata(alias="accountingField2"),
        pydantic.Field(alias="accountingField2"),
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString],
        FieldMetadata(alias="additionalData"),
        pydantic.Field(alias="additionalData"),
    ] = None
    attachments: typing.Optional[Attachments] = pydantic.Field(default=None)
    """
    Bill image attachment. Send the bill image as Base64-encoded string, or as a publicly accessible link. For full details on using this field with a payout authorization, see [the documentation](/developers/developer-guides/pay-out-manage-payouts).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .page_content import PageContent
from .page_identifier import PageIdentifier
from .page_setting import PageSetting
from .payabli_credentials import PayabliCredentials
from .receipt_content import ReceiptContent
from .subdomain import Subdomain


class PayabliPages(UniversalBaseModel):
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="AdditionalData"), pydantic.Field(alias="AdditionalData")
    ] = None
    credentials: typing.Optional[typing.List[PayabliCredentials]] = pydantic.Field(default=None)
    """
    Array of credential objects with active services for the page
    """

    last_access: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastAccess"),
        pydantic.Field(alias="lastAccess", description="Timestamp of last access to page structure"),
    ] = None
    page_content: typing_extensions.Annotated[
        typing.Optional[PageContent],
        FieldMetadata(alias="pageContent"),
        pydantic.Field(alias="pageContent", description="Sections of page"),
    ] = None
    page_identifier: typing_extensions.Annotated[
        typing.Optional[PageIdentifier], FieldMetadata(alias="pageIdentifier"), pydantic.Field(alias="pageIdentifier")
    ] = None
    page_settings: typing_extensions.Annotated[
        typing.Optional[PageSetting],
        FieldMetadata(alias="pageSettings"),
        pydantic.Field(alias="pageSettings", description="Settings of page"),
    ] = None
    published: typing.Optional[int] = pydantic.Field(default=None)
    """
    Flag indicating if page is active to accept payments. `0` for false, `1` for true.
    """

    receipt_content: typing_extensions.Annotated[
        typing.Optional[ReceiptContent],
        FieldMetadata(alias="receiptContent"),
        pydantic.Field(alias="receiptContent", description="Sections of payment receipt"),
    ] = None
    subdomain: typing.Optional[Subdomain] = pydantic.Field(default=None)
    """
    Page identifier. Must be unique in platform.
    """

    total_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="totalAmount"),
        pydantic.Field(alias="totalAmount", description="Total amount to pay in this page"),
    ] = None
    validation_code: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="validationCode"),
        pydantic.Field(
            alias="validationCode", description="Base64 encoded image of CAPTCHA associated to this page load"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...money_in.types.transaction_detail_customer import TransactionDetailCustomer
from ...money_in.types.transaction_detail_event import TransactionDetailEvent
from ...money_in.types.transaction_detail_invoice_data import TransactionDetailInvoiceData
from ...money_in.types.transaction_detail_payment_data import TransactionDetailPaymentData
from ...types.ach_holder_type import AchHolderType
from ...types.ach_sec_code import AchSecCode
from ...types.batch_number import BatchNumber
from ...types.chargeback_id import ChargebackId
from ...types.dbaname import Dbaname
from ...types.device import Device
from ...types.entrypage_id import EntrypageId
from ...types.entrypointfield import Entrypointfield
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.external_processor_information import ExternalProcessorInformation
from ...types.fee_amount import FeeAmount
from ...types.ip_address import IpAddress
from ...types.legalname import Legalname
from ...types.operation import Operation
from ...types.order_id import OrderId
from ...types.org_parent_name import OrgParentName
from ...types.orgid import Orgid
from ...types.payor_id import PayorId
from ...types.paypoint_id import PaypointId
from ...types.pending_fee_amount import PendingFeeAmount
from ...types.query_c_fee_transaction import QueryCFeeTransaction
from ...types.refund_id import RefundId
from ...types.retrieval_id import RetrievalId
from ...types.returned_id import ReturnedId
from ...types.risk_action import RiskAction
from ...types.risk_action_code import RiskActionCode
from ...types.risk_flagged import RiskFlagged
from ...types.risk_flagged_on import RiskFlaggedOn
from ...types.risk_reason import RiskReason
from ...types.risk_status import RiskStatus
from ...types.settlement_status import SettlementStatus
from ...types.source import Source
from ...types.split_funding import SplitFunding
from ...types.trans_status import TransStatus
from .v_2_transaction_detail_response_data import V2TransactionDetailResponseData


class V2TransactionDetails(UniversalBaseModel):
    """
    Complete transaction details returned by v2 Money In endpoints. This matches the structure of the transaction details previously returned by the v1 details endpoint.

    Examples
    --------
    import datetime

    from payabli import BinData, QueryCFeeTransaction
    from payabli.money_in import (
        TransactionDetailCustomer,
        TransactionDetailEvent,
        TransactionDetailInvoiceData,
        TransactionDetailPaymentData,
        TransactionDetailPaymentDetails,
    )
    from payabli.v_2_money_in_types import (
        V2TransactionDetailResponseData,
        V2TransactionDetails,
    )

    V2TransactionDetails(
        parent_org_name="Mrinal's Pet Supplies",
        paypoint_dbaname="Mrinal's Pet Shop North",
        paypoint_legalname="Mrinal's Pet Shop North",
        paypoint_entryname="495147f647",
        payment_trans_id="3040-96dfa9a7c4ed4f82a3dd4a4a12ad28ae",
        connector_name="gp",
        external_processor_information="",
        gateway_trans_id="TRN_Ih68D6UZdip7OEQ2QFXat1yQSLF2nB",
        method="card",
        batch_number="3040_combined_20251201_3a50747d-6b5c-40ef-9f69-93a9cc7fcb49",
        batch_amount=420.0,
        payor_id=40144,
        payment_data=TransactionDetailPaymentData(
            masked_account="4XXXXXXXXXXX5439",
            account_type="visa",
            account_exp="12/25",
            holder_name="John Cassian",
            order_description="",
            bin_data=BinData(
                bin_matched_length="6",
                bin_card_brand="VISA",
                bin_card_type="CREDIT",
                bin_card_category="CLASSIC",
                bin_card_issuer="",
                bin_card_issuer_country="RUSSIAN FEDERATION",
                bin_card_issuer_country_code_a_2="RU",
                bin_card_issuer_country_number="643",
                bin_card_is_regulated="",
                bin_card_use_category="",
                bin_card_issuer_country_code_a_3="",
            ),
            payment_details=TransactionDetailPaymentDetails(
                total_amount=105.0,
                service_fee=5.0,
                check_unique_id="",
                currency="USD",
                categories=[],
                split_funding=[],
            ),
        ),
        trans_status=1,
        paypoint_id=3040,
        total_amount=105.0,
        net_amount=100.0,
        fee_amount=5.0,
        settlement_status=0,
        operation="Sale",
        response_data=V2TransactionDetailResponseData(
            result_code="A0000",
            result_code_text="Approved",
            responsetext="CAPTURED",
            authcode="AXS425",
            transactionid="TRN_Xo4dpKfmx3OxSc9svd2ccI6OOnyB2I",
            avsresponse="N",
            avsresponse_text="No Match, No address or ZIP match",
            cvvresponse="M",
            cvvresponse_text="CVV2/CVC2 match",
            response_code="100",
            response_code_text="Operation successful",
        ),
        source="api",
        schedule_reference=0,
        org_id=17448,
        refund_id=0,
        returned_id=0,
        chargeback_id=0,
        retrieval_id=0,
        invoice_data=TransactionDetailInvoiceData(),
        entrypage_id=0,
        external_paypoint_id="",
        is_validated_ach=False,
        transaction_time="2025-12-01T09:50:03.559",
        customer=TransactionDetailCustomer(
            first_name="David",
            last_name="Beckham",
            company_name="Driving School LLC",
            billing_address_1="Home Address",
            billing_address_2="",
            billing_city="",
            billing_state="",
            billing_zip="45157",
            billing_country="US",
            billing_phone="+15555555555",
            billing_email="mrinal.kundu@payabli.com",
            customer_number="C-90010",
            shipping_address_1="Home Address",
            shipping_address_2="",
            shipping_city="",
            shipping_state="",
            shipping_zip="45157",
            shipping_country="US",
            customer_id=40144,
            customer_status=0,
        ),
        cfee_transactions=[
            QueryCFeeTransaction(
                c_fee_transid="3040-96dfa9a7c4ed4f82a3dd4a4a12ad28ae",
                trans_status=1,
                fee_amount=5.0,
                settlement_status=0,
                operation="Sale",
                response_data={},
                refund_id=0,
                transaction_time=datetime.datetime.fromisoformat(
                    "2025-12-01 09:50:03.559000+00:00",
                ),
            )
        ],
        transaction_events=[
            TransactionDetailEvent(
                trans_event="Created",
                event_data="0HNHD68HATSUC:00000001",
                event_time="2025-12-01T09:50:02.558651",
            ),
            TransactionDetailEvent(
                trans_event="Approved",
                event_data="0HNHD68HATSUC:00000001",
                event_time="2025-12-01T09:50:03.609111",
            ),
        ],
        pending_fee_amount=0.0,
        risk_flagged=False,
        risk_flagged_on=datetime.datetime.fromisoformat(
            "2025-12-01 09:50:02.547000+00:00",
        ),
        risk_status="PASSED",
        risk_reason="",
        risk_action="",
        risk_action_code=0,
        device_id="",
        ach_sec_code="",
        ach_holder_type="personal",
        ip_address="255.255.255.255",
        is_same_day_ach=False,
    )
    """

    parent_org_name: typing_extensions.Annotated[
        OrgParentName, FieldMetadata(alias="parentOrgName"), pydantic.Field(alias="parentOrgName")
    ]
    paypoint_dbaname: typing_extensions.Annotated[
        Dbaname, FieldMetadata(alias="paypointDbaname"), pydantic.Field(alias="paypointDbaname")
    ]
    paypoint_legalname: typing_extensions.Annotated[
        Legalname, FieldMetadata(alias="paypointLegalname"), pydantic.Field(alias="paypointLegalname")
    ]
    paypoint_entryname: typing_extensions.Annotated[
        Entrypointfield, FieldMetadata(alias="paypointEntryname"), pydantic.Field(alias="paypointEntryname")
    ]
    payment_trans_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="paymentTransId"),
        pydantic.Field(alias="paymentTransId", description="Unique transaction identifier."),
    ]
    connector_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="connectorName"),
        pydantic.Field(alias="connectorName", description="Name of the payment connector used."),
    ]
    external_processor_information: typing_extensions.Annotated[
        ExternalProcessorInformation,
        FieldMetadata(alias="externalProcessorInformation"),
        pydantic.Field(alias="externalProcessorInformation"),
    ]
    gateway_trans_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="gatewayTransId"),
        pydantic.Field(alias="gatewayTransId", description="Gateway transaction identifier."),
    ]
    order_id: typing_extensions.Annotated[
        typing.Optional[OrderId], FieldMetadata(alias="orderId"), pydantic.Field(alias="orderId")
    ] = None
    method: str = pydantic.Field()
    """
    Payment method used for the transaction.
    """

    batch_number: typing_extensions.Annotated[
        BatchNumber, FieldMetadata(alias="batchNumber"), pydantic.Field(alias="batchNumber")
    ]
    batch_amount: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="batchAmount"),
        pydantic.Field(alias="batchAmount", description="Total amount in the batch."),
    ]
    payor_id: typing_extensions.Annotated[PayorId, FieldMetadata(alias="payorId"), pydantic.Field(alias="payorId")]
    payment_data: typing_extensions.Annotated[
        TransactionDetailPaymentData, FieldMetadata(alias="paymentData"), pydantic.Field(alias="paymentData")
    ]
    trans_status: typing_extensions.Annotated[
        TransStatus, FieldMetadata(alias="transStatus"), pydantic.Field(alias="transStatus")
    ]
    paypoint_id: typing_extensions.Annotated[
        PaypointId, FieldMetadata(alias="paypointId"), pydantic.Field(alias="paypointId")
    ]
    total_amount: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="totalAmount"),
        pydantic.Field(alias="totalAmount", description="Total transaction amount including fees."),
    ]
    net_amount: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="netAmount"),
        pydantic.Field(alias="netAmount", description="Net transaction amount excluding fees."),
    ]
    fee_amount: typing_extensions.Annotated[
        FeeAmount, FieldMetadata(alias="feeAmount"), pydantic.Field(alias="feeAmount")
    ]
    settlement_status: typing_extensions.Annotated[
        SettlementStatus, FieldMetadata(alias="settlementStatus"), pydantic.Field(alias="settlementStatus")
    ]
    operation: Operation
    response_data: typing_extensions.Annotated[
        V2TransactionDetailResponseData, FieldMetadata(alias="responseData"), pydantic.Field(alias="responseData")
    ]
    source: Source
    schedule_reference: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="scheduleReference"),
        pydantic.Field(
            alias="scheduleReference", description="Reference to associated payment schedule if applicable."
        ),
    ]
    org_id: typing_extensions.Annotated[Orgid, FieldMetadata(alias="orgId"), pydantic.Field(alias="orgId")]
    refund_id: typing_extensions.Annotated[RefundId, FieldMetadata(alias="refundId"), pydantic.Field(alias="refundId")]
    returned_id: typing_extensions.Annotated[
        ReturnedId, FieldMetadata(alias="returnedId"), pydantic.Field(alias="returnedId")
    ]
    chargeback_id: typing_extensions.Annotated[
        ChargebackId, FieldMetadata(alias="chargebackId"), pydantic.Field(alias="chargebackId")
    ]
    retrieval_id: typing_extensions.Annotated[
        RetrievalId, FieldMetadata(alias="retrievalId"), pydantic.Field(alias="retrievalId")
    ]
    trans_additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Any],
        FieldMetadata(alias="transAdditionalData"),
        pydantic.Field(alias="transAdditionalData"),
    ] = None
    invoice_data: typing_extensions.Annotated[
        TransactionDetailInvoiceData, FieldMetadata(alias="invoiceData"), pydantic.Field(alias="invoiceData")
    ]
    entrypage_id: typing_extensions.Annotated[
        EntrypageId, FieldMetadata(alias="entrypageId"), pydantic.Field(alias="entrypageId")
    ]
    external_paypoint_id: typing_extensions.Annotated[
        ExternalPaypointId, FieldMetadata(alias="externalPaypointID"), pydantic.Field(alias="externalPaypointID")
    ]
    is_validated_ach: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isValidatedACH"),
        pydantic.Field(alias="isValidatedACH", description="Indicates if ACH account was validated in real-time."),
    ]
    transaction_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="transactionTime"),
        pydantic.Field(alias="transactionTime", description="Timestamp when transaction was created."),
    ]
    customer: TransactionDetailCustomer
    split_funding_instructions: typing_extensions.Annotated[
        typing.Optional[SplitFunding],
        FieldMetadata(alias="splitFundingInstructions"),
        pydantic.Field(alias="splitFundingInstructions"),
    ] = None
    cfee_transactions: typing_extensions.Annotated[
        typing.List[QueryCFeeTransaction],
        FieldMetadata(alias="cfeeTransactions"),
        pydantic.Field(alias="cfeeTransactions"),
    ]
    transaction_events: typing_extensions.Annotated[
        typing.List[TransactionDetailEvent],
        FieldMetadata(alias="transactionEvents"),
        pydantic.Field(alias="transactionEvents"),
    ]
    pending_fee_amount: typing_extensions.Annotated[
        PendingFeeAmount, FieldMetadata(alias="pendingFeeAmount"), pydantic.Field(alias="pendingFeeAmount")
    ]
    risk_flagged: typing_extensions.Annotated[
        RiskFlagged, FieldMetadata(alias="riskFlagged"), pydantic.Field(alias="riskFlagged")
    ]
    risk_flagged_on: typing_extensions.Annotated[
        RiskFlaggedOn, FieldMetadata(alias="riskFlaggedOn"), pydantic.Field(alias="riskFlaggedOn")
    ]
    risk_status: typing_extensions.Annotated[
        RiskStatus, FieldMetadata(alias="riskStatus"), pydantic.Field(alias="riskStatus")
    ]
    risk_reason: typing_extensions.Annotated[
        RiskReason, FieldMetadata(alias="riskReason"), pydantic.Field(alias="riskReason")
    ]
    risk_action: typing_extensions.Annotated[
        RiskAction, FieldMetadata(alias="riskAction"), pydantic.Field(alias="riskAction")
    ]
    risk_action_code: typing_extensions.Annotated[
        RiskActionCode, FieldMetadata(alias="riskActionCode"), pydantic.Field(alias="riskActionCode")
    ]
    device_id: typing_extensions.Annotated[Device, FieldMetadata(alias="deviceId"), pydantic.Field(alias="deviceId")]
    ach_sec_code: typing_extensions.Annotated[
        AchSecCode, FieldMetadata(alias="achSecCode"), pydantic.Field(alias="achSecCode")
    ]
    ach_holder_type: typing_extensions.Annotated[
        AchHolderType, FieldMetadata(alias="achHolderType"), pydantic.Field(alias="achHolderType")
    ]
    ip_address: typing_extensions.Annotated[
        IpAddress, FieldMetadata(alias="ipAddress"), pydantic.Field(alias="ipAddress")
    ]
    is_same_day_ach: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isSameDayACH"),
        pydantic.Field(alias="isSameDayACH", description="Indicates if ACH transaction uses same-day processing."),
    ]
    wallet_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="walletType"),
        pydantic.Field(alias="walletType", description="Digital wallet type if applicable."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .bill_id import BillId
from .comments import Comments
from .datenullable import Datenullable
from .invoice_number import InvoiceNumber
from .net_amountstring import NetAmountstring


class BillDetailsResponse(UniversalBaseModel):
    """
    Response object for bill details. Contains basic information about a bill.
    """

    bill_id: typing_extensions.Annotated[
        typing.Optional[BillId], FieldMetadata(alias="billId"), pydantic.Field(alias="billId")
    ] = None
    lot_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="lotNumber"),
        pydantic.Field(alias="lotNumber", description="Lot number of the bill."),
    ] = None
    invoice_number: typing_extensions.Annotated[
        typing.Optional[InvoiceNumber],
        FieldMetadata(alias="invoiceNumber"),
        pydantic.Field(alias="invoiceNumber", description="Custom number identifying the bill."),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[NetAmountstring],
        FieldMetadata(alias="netAmount"),
        pydantic.Field(alias="netAmount", description="Net Amount owed in bill. Required when adding a bill."),
    ] = None
    discount: typing.Optional[str] = pydantic.Field(default=None)
    """
    Bill discount amount.
    """

    due_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="dueDate"),
        pydantic.Field(alias="dueDate", description="Bill due date in format YYYY-MM-DD or MM/DD/YYYY."),
    ] = None
    invoice_date: typing_extensions.Annotated[
        typing.Optional[Datenullable],
        FieldMetadata(alias="invoiceDate"),
        pydantic.Field(alias="invoiceDate", description="Bill date in format YYYY-MM-DD or MM/DD/YYYY."),
    ] = None
    comments: typing.Optional[Comments] = pydantic.Field(default=None)
    """
    Any comments about bill. **For managed payouts, this field has a limit of 100 characters**.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .transfer_out_detail_bill_attachment import TransferOutDetailBillAttachment


class TransferOutDetailBill(UniversalBaseModel):
    """
    Bill information for an outbound transfer detail.
    """

    bill_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="billId"),
        pydantic.Field(alias="billId", description="Unique identifier for the bill."),
    ] = None
    lot_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="LotNumber"),
        pydantic.Field(alias="LotNumber", description="Lot number."),
    ] = None
    accounting_field_1: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="AccountingField1"),
        pydantic.Field(alias="AccountingField1", description="Accounting field 1."),
    ] = None
    accounting_field_2: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="AccountingField2"),
        pydantic.Field(alias="AccountingField2", description="Accounting field 2."),
    ] = None
    terms: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="Terms"), pydantic.Field(alias="Terms", description="Payment terms.")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Any]],
        FieldMetadata(alias="AdditionalData"),
        pydantic.Field(alias="AdditionalData", description="Additional data for the bill."),
    ] = None
    attachments: typing.Optional[typing.List[TransferOutDetailBillAttachment]] = pydantic.Field(default=None)
    """
    Attachments for the bill.
    """

    invoice_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="invoiceNumber"),
        pydantic.Field(alias="invoiceNumber", description="Invoice number."),
    ] = None
    net_amount: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="netAmount"),
        pydantic.Field(alias="netAmount", description="Net amount of the bill."),
    ] = None
    invoice_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="invoiceDate"),
        pydantic.Field(alias="invoiceDate", description="Date of the invoice."),
    ] = None
    due_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dueDate"),
        pydantic.Field(alias="dueDate", description="Due date for the bill."),
    ] = None
    comments: typing.Optional[str] = pydantic.Field(default=None)
    """
    Comments on the bill.
    """

    identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifier for the bill.
    """

    discount: typing.Optional[float] = pydantic.Field(default=None)
    """
    Discount applied.
    """

    total_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="totalAmount"),
        pydantic.Field(alias="totalAmount", description="Total amount of the bill."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_map import AdditionalDataMap
from .billing_zip import BillingZip
from .customer_id import CustomerId
from .customer_number_nullable import CustomerNumberNullable
from .customer_status import CustomerStatus
from .email import Email
from .shippingaddress import Shippingaddress
from .shippingaddressadditional import Shippingaddressadditional
from .shippingcity import Shippingcity
from .shippingcountry import Shippingcountry
from .shippingstate import Shippingstate
from .shippingzip import Shippingzip


class QueryTransactionPayorDataCustomer(UniversalBaseModel):
    identifiers: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Any]],
        FieldMetadata(alias="Identifiers"),
        pydantic.Field(alias="Identifiers", description="Array of field names to be used as identifiers."),
    ] = None
    first_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="FirstName"),
        pydantic.Field(alias="FirstName", description="Customer/Payor first name."),
    ] = None
    last_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="LastName"),
        pydantic.Field(alias="LastName", description="Customer/Payor last name."),
    ] = None
    company_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="CompanyName"),
        pydantic.Field(alias="CompanyName", description="Customer's company name."),
    ] = None
    billing_address_1: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillingAddress1"),
        pydantic.Field(alias="BillingAddress1", description="Customer's billing address."),
    ] = None
    billing_address_2: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillingAddress2"),
        pydantic.Field(alias="BillingAddress2", description="Additional line for Customer's billing address."),
    ] = None
    billing_city: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillingCity"),
        pydantic.Field(alias="BillingCity", description="Customer's billing city."),
    ] = None
    billing_state: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillingState"),
        pydantic.Field(
            alias="BillingState", description="Customer's billing state. Must be 2-letter state code for address in US."
        ),
    ] = None
    billing_zip: typing_extensions.Annotated[
        typing.Optional[BillingZip],
        FieldMetadata(alias="BillingZip"),
        pydantic.Field(alias="BillingZip", description="Customer's billing ZIP code."),
    ] = None
    billing_country: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillingCountry"),
        pydantic.Field(alias="BillingCountry", description="Customer's billing country."),
    ] = None
    billing_phone: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="BillingPhone"),
        pydantic.Field(alias="BillingPhone", description="Customer's phone number."),
    ] = None
    billing_email: typing_extensions.Annotated[
        typing.Optional[Email],
        FieldMetadata(alias="BillingEmail"),
        pydantic.Field(alias="BillingEmail", description="Customer's email address."),
    ] = None
    customer_number: typing_extensions.Annotated[
        typing.Optional[CustomerNumberNullable],
        FieldMetadata(alias="CustomerNumber"),
        pydantic.Field(alias="CustomerNumber"),
    ] = None
    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddress],
        FieldMetadata(alias="ShippingAddress1"),
        pydantic.Field(alias="ShippingAddress1"),
    ] = None
    shipping_address_2: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional],
        FieldMetadata(alias="ShippingAddress2"),
        pydantic.Field(alias="ShippingAddress2"),
    ] = None
    shipping_city: typing_extensions.Annotated[
        typing.Optional[Shippingcity], FieldMetadata(alias="ShippingCity"), pydantic.Field(alias="ShippingCity")
    ] = None
    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="ShippingState"), pydantic.Field(alias="ShippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[
        typing.Optional[Shippingzip], FieldMetadata(alias="ShippingZip"), pydantic.Field(alias="ShippingZip")
    ] = None
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry],
        FieldMetadata(alias="ShippingCountry"),
        pydantic.Field(alias="ShippingCountry"),
    ] = None
    customer_id: typing_extensions.Annotated[
        typing.Optional[CustomerId], FieldMetadata(alias="customerId"), pydantic.Field(alias="customerId")
    ] = None
    customer_status: typing_extensions.Annotated[
        typing.Optional[CustomerStatus], FieldMetadata(alias="customerStatus"), pydantic.Field(alias="customerStatus")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataMap],
        FieldMetadata(alias="AdditionalData"),
        pydantic.Field(alias="AdditionalData"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .item_commodity_code import ItemCommodityCode
from .item_description import ItemDescription
from .item_product_code import ItemProductCode
from .item_product_name import ItemProductName
from .item_unitof_measure import ItemUnitofMeasure


class LineItem(UniversalBaseModel):
    item_categories: typing_extensions.Annotated[
        typing.Optional[typing.List[typing.Optional[str]]],
        FieldMetadata(alias="itemCategories"),
        pydantic.Field(alias="itemCategories", description="Array of tags classifying item or product."),
    ] = None
    item_commodity_code: typing_extensions.Annotated[
        typing.Optional[ItemCommodityCode],
        FieldMetadata(alias="itemCommodityCode"),
        pydantic.Field(alias="itemCommodityCode"),
    ] = None
    item_cost: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="itemCost"),
        pydantic.Field(alias="itemCost", description="Item or product price per unit."),
    ]
    item_description: typing_extensions.Annotated[
        typing.Optional[ItemDescription],
        FieldMetadata(alias="itemDescription"),
        pydantic.Field(alias="itemDescription"),
    ] = None
    item_mode: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="itemMode"),
        pydantic.Field(
            alias="itemMode",
            description="Internal class of item or product: value '0' is only for invoices, '1' for bills, and '2' is common for both.",
        ),
    ] = None
    item_product_code: typing_extensions.Annotated[
        typing.Optional[ItemProductCode],
        FieldMetadata(alias="itemProductCode"),
        pydantic.Field(alias="itemProductCode"),
    ] = None
    item_product_name: typing_extensions.Annotated[
        typing.Optional[ItemProductName],
        FieldMetadata(alias="itemProductName"),
        pydantic.Field(alias="itemProductName"),
    ] = None
    item_qty: typing_extensions.Annotated[
        int, FieldMetadata(alias="itemQty"), pydantic.Field(alias="itemQty", description="Quantity of item or product.")
    ]
    item_unit_of_measure: typing_extensions.Annotated[
        typing.Optional[ItemUnitofMeasure],
        FieldMetadata(alias="itemUnitOfMeasure"),
        pydantic.Field(alias="itemUnitOfMeasure"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.entrypointfield import Entrypointfield
from .notification_request import NotificationRequest


class PaypointMoveRequest(UniversalBaseModel):
    entry_point: typing_extensions.Annotated[
        Entrypointfield, FieldMetadata(alias="entryPoint"), pydantic.Field(alias="entryPoint")
    ]
    new_parent_organization_id: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="newParentOrganizationId"),
        pydantic.Field(
            alias="newParentOrganizationId", description="The ID for the paypoint's new parent organization."
        ),
    ]
    notification_request: typing_extensions.Annotated[
        typing.Optional[NotificationRequest],
        FieldMetadata(alias="notificationRequest"),
        pydantic.Field(alias="notificationRequest", description="Optional notification request object for a webhook"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PaymentDetailCredit(UniversalBaseModel):
    """
    The PaymentDetail object for microdeposit (MakeCredit) transactions.
    """

    currency: typing.Optional[str] = pydantic.Field(default=None)
    """
    Currency code ISO-4217. If not code is provided the currency in the paypoint setting is taken. Default is **USD**
    """

    service_fee: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="serviceFee"),
        pydantic.Field(
            alias="serviceFee",
            description="Service fee to be deducted from the total amount. This amount must be a number, percentages aren't accepted. If you are using a percentage-based fee schedule, you must calculate the value manually.",
        ),
    ] = None
    total_amount: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="totalAmount"),
        pydantic.Field(
            alias="totalAmount",
            description="Total amount to be charged. If a service fee is provided, then this amount should include the service fee.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class BillingFeeDetail(UniversalBaseModel):
    billable_event: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="billableEvent"), pydantic.Field(alias="billableEvent")
    ] = None
    service: typing.Optional[str] = None
    event_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="eventId"), pydantic.Field(alias="eventId")
    ] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the billing fee
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    Category of the billing fee
    """

    fix_price: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="fixPrice"),
        pydantic.Field(alias="fixPrice", description="Fixed price component of the fee"),
    ] = None
    float_price: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="floatPrice"),
        pydantic.Field(alias="floatPrice", description="Percentage component of the fee"),
    ] = None
    billable_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="billableAmount"),
        pydantic.Field(alias="billableAmount", description="Amount eligible for the fee"),
    ] = None
    bill_amount: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="billAmount"),
        pydantic.Field(alias="billAmount", description="Total fee amount charged"),
    ] = None
    frequency: typing.Optional[str] = None
    service_group: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="serviceGroup"), pydantic.Field(alias="serviceGroup")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.additional_data_string import AdditionalDataString
from ...types.attachments import Attachments
from ...types.bill_item import BillItem
from ...types.datenullable import Datenullable
from ...types.discount import Discount
from ...types.duty_amount import DutyAmount
from ...types.email import Email
from ...types.freight_amount import FreightAmount
from ...types.frequency import Frequency
from ...types.invoice_amount import InvoiceAmount
from ...types.invoice_number import InvoiceNumber
from ...types.invoice_type import InvoiceType
from ...types.invoicestatus import Invoicestatus
from ...types.purchase_order import PurchaseOrder
from ...types.shipping_from_zip import ShippingFromZip
from ...types.shippingaddress import Shippingaddress
from ...types.shippingaddressadditional import Shippingaddressadditional
from ...types.shippingcity import Shippingcity
from ...types.shippingcountry import Shippingcountry
from ...types.shippingstate import Shippingstate
from ...types.shippingzip import Shippingzip
from ...types.summary_commodity_code import SummaryCommodityCode
from ...types.tax import Tax
from ...types.terms_conditions import TermsConditions


class TransactionDetailInvoiceData(UniversalBaseModel):
    """
    Invoice information if transaction is associated with an invoice
    """

    invoice_number: typing_extensions.Annotated[
        typing.Optional[InvoiceNumber], FieldMetadata(alias="invoiceNumber"), pydantic.Field(alias="invoiceNumber")
    ] = None
    invoice_date: typing_extensions.Annotated[
        typing.Optional[Datenullable], FieldMetadata(alias="invoiceDate"), pydantic.Field(alias="invoiceDate")
    ] = None
    invoice_due_date: typing_extensions.Annotated[
        typing.Optional[Datenullable], FieldMetadata(alias="invoiceDueDate"), pydantic.Field(alias="invoiceDueDate")
    ] = None
    invoice_end_date: typing_extensions.Annotated[
        typing.Optional[Datenullable], FieldMetadata(alias="invoiceEndDate"), pydantic.Field(alias="invoiceEndDate")
    ] = None
    invoice_status: typing_extensions.Annotated[
        typing.Optional[Invoicestatus], FieldMetadata(alias="invoiceStatus"), pydantic.Field(alias="invoiceStatus")
    ] = None
    invoice_type: typing_extensions.Annotated[
        typing.Optional[InvoiceType], FieldMetadata(alias="invoiceType"), pydantic.Field(alias="invoiceType")
    ] = None
    frequency: typing.Optional[Frequency] = None
    payment_terms: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="paymentTerms"), pydantic.Field(alias="paymentTerms")
    ] = None
    terms_conditions: typing_extensions.Annotated[
        typing.Optional[TermsConditions],
        FieldMetadata(alias="termsConditions"),
        pydantic.Field(alias="termsConditions"),
    ] = None
    notes: typing.Optional[str] = None
    tax: typing.Optional[Tax] = None
    discount: typing.Optional[Discount] = None
    invoice_amount: typing_extensions.Annotated[
        typing.Optional[InvoiceAmount], FieldMetadata(alias="invoiceAmount"), pydantic.Field(alias="invoiceAmount")
    ] = None
    freight_amount: typing_extensions.Annotated[
        typing.Optional[FreightAmount], FieldMetadata(alias="freightAmount"), pydantic.Field(alias="freightAmount")
    ] = None
    duty_amount: typing_extensions.Annotated[
        typing.Optional[DutyAmount], FieldMetadata(alias="dutyAmount"), pydantic.Field(alias="dutyAmount")
    ] = None
    purchase_order: typing_extensions.Annotated[
        typing.Optional[PurchaseOrder], FieldMetadata(alias="purchaseOrder"), pydantic.Field(alias="purchaseOrder")
    ] = None
    first_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="firstName"), pydantic.Field(alias="firstName")
    ] = None
    last_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="lastName"), pydantic.Field(alias="lastName")
    ] = None
    company: typing.Optional[str] = None
    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddress],
        FieldMetadata(alias="shippingAddress1"),
        pydantic.Field(alias="shippingAddress1"),
    ] = None
    shipping_address_2: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional],
        FieldMetadata(alias="shippingAddress2"),
        pydantic.Field(alias="shippingAddress2"),
    ] = None
    shipping_city: typing_extensions.Annotated[
        typing.Optional[Shippingcity], FieldMetadata(alias="shippingCity"), pydantic.Field(alias="shippingCity")
    ] = None
    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="shippingState"), pydantic.Field(alias="shippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[
        typing.Optional[Shippingzip], FieldMetadata(alias="shippingZip"), pydantic.Field(alias="shippingZip")
    ] = None
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry],
        FieldMetadata(alias="shippingCountry"),
        pydantic.Field(alias="shippingCountry"),
    ] = None
    shipping_email: typing_extensions.Annotated[
        typing.Optional[Email], FieldMetadata(alias="shippingEmail"), pydantic.Field(alias="shippingEmail")
    ] = None
    shipping_phone: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="shippingPhone"), pydantic.Field(alias="shippingPhone")
    ] = None
    shipping_from_zip: typing_extensions.Annotated[
        typing.Optional[ShippingFromZip],
        FieldMetadata(alias="shippingFromZip"),
        pydantic.Field(alias="shippingFromZip"),
    ] = None
    summary_commodity_code: typing_extensions.Annotated[
        typing.Optional[SummaryCommodityCode],
        FieldMetadata(alias="summaryCommodityCode"),
        pydantic.Field(alias="summaryCommodityCode"),
    ] = None
    items: typing.Optional[typing.List[BillItem]] = None
    attachments: typing.Optional[Attachments] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString],
        FieldMetadata(alias="additionalData"),
        pydantic.Field(alias="additionalData"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_string import AdditionalDataString
from .created_at import CreatedAt
from .email import Email
from .language import Language
from .mfa import Mfa
from .mfa_mode import MfaMode
from .name_user import NameUser
from .org_x_scope import OrgXScope
from .phone_number import PhoneNumber
from .timezone import Timezone
from .usr_access import UsrAccess
from .usr_status import UsrStatus


class UserQueryRecord(UniversalBaseModel):
    access: typing_extensions.Annotated[
        typing.Optional[typing.List[UsrAccess]], FieldMetadata(alias="Access"), pydantic.Field(alias="Access")
    ] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString],
        FieldMetadata(alias="AdditionalData"),
        pydantic.Field(alias="AdditionalData"),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[CreatedAt],
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The timestamp for the user's creation, in UTC."),
    ] = None
    email: typing_extensions.Annotated[
        typing.Optional[Email],
        FieldMetadata(alias="Email"),
        pydantic.Field(alias="Email", description="The user's email address."),
    ] = None
    language: typing.Optional[Language] = None
    last_access: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastAccess"),
        pydantic.Field(alias="lastAccess", description="The timestamp for the user's last activity, in UTC."),
    ] = None
    name: typing_extensions.Annotated[
        typing.Optional[NameUser], FieldMetadata(alias="Name"), pydantic.Field(alias="Name")
    ] = None
    phone: typing_extensions.Annotated[
        typing.Optional[PhoneNumber],
        FieldMetadata(alias="Phone"),
        pydantic.Field(alias="Phone", description="The user's phone number."),
    ] = None
    scope: typing_extensions.Annotated[
        typing.Optional[typing.List[OrgXScope]], FieldMetadata(alias="Scope"), pydantic.Field(alias="Scope")
    ] = None
    sn_data: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snData"),
        pydantic.Field(
            alias="snData", description="Additional data provided by the social network related to the customer."
        ),
    ] = None
    sn_identifier: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snIdentifier"),
        pydantic.Field(alias="snIdentifier", description="Identifier or token for customer in linked social network."),
    ] = None
    sn_provider: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="snProvider"),
        pydantic.Field(
            alias="snProvider",
            description="Social network linked to customer. Possible values: facebook, google, twitter, microsoft.",
        ),
    ] = None
    time_zone: typing_extensions.Annotated[
        typing.Optional[Timezone], FieldMetadata(alias="timeZone"), pydantic.Field(alias="timeZone")
    ] = None
    user_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="userId"),
        pydantic.Field(alias="userId", description="The user's ID in Payabli."),
    ] = None
    usr_mfa: typing_extensions.Annotated[
        typing.Optional[Mfa], FieldMetadata(alias="UsrMFA"), pydantic.Field(alias="UsrMFA")
    ] = None
    usr_mfa_mode: typing_extensions.Annotated[
        typing.Optional[MfaMode], FieldMetadata(alias="UsrMFAMode"), pydantic.Field(alias="UsrMFAMode")
    ] = None
    usr_status: typing_extensions.Annotated[
        typing.Optional[UsrStatus], FieldMetadata(alias="UsrStatus"), pydantic.Field(alias="UsrStatus")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .boarding_query_links import BoardingQueryLinks
from .created_at import CreatedAt
from .is_root import IsRoot
from .org_parent_name import OrgParentName
from .recipient_email_notification import RecipientEmailNotification
from .resumable import Resumable
from .template_code import TemplateCode
from .template_content_response import TemplateContentResponse


class TemplateQueryRecord(UniversalBaseModel):
    add_price: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="addPrice"), pydantic.Field(alias="addPrice")
    ] = None
    boarding_links: typing_extensions.Annotated[
        typing.Optional[typing.List[BoardingQueryLinks]],
        FieldMetadata(alias="boardingLinks"),
        pydantic.Field(alias="boardingLinks"),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[CreatedAt], FieldMetadata(alias="createdAt"), pydantic.Field(alias="createdAt")
    ] = None
    id_template: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="idTemplate"), pydantic.Field(alias="idTemplate")
    ] = None
    is_root: typing_extensions.Annotated[
        typing.Optional[IsRoot], FieldMetadata(alias="isRoot"), pydantic.Field(alias="isRoot")
    ] = None
    org_parent_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="orgParentName"), pydantic.Field(alias="orgParentName")
    ] = None
    recipient_email_notification: typing_extensions.Annotated[
        typing.Optional[RecipientEmailNotification],
        FieldMetadata(alias="recipientEmailNotification"),
        pydantic.Field(alias="recipientEmailNotification"),
    ] = None
    resumable: typing.Optional[Resumable] = None
    template_code: typing_extensions.Annotated[
        typing.Optional[TemplateCode], FieldMetadata(alias="templateCode"), pydantic.Field(alias="templateCode")
    ] = None
    template_content: typing_extensions.Annotated[
        typing.Optional[TemplateContentResponse],
        FieldMetadata(alias="templateContent"),
        pydantic.Field(alias="templateContent"),
    ] = None
    template_description: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="templateDescription"), pydantic.Field(alias="templateDescription")
    ] = None
    template_title: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="templateTitle"), pydantic.Field(alias="templateTitle")
    ] = None
    used_by: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="usedBy"), pydantic.Field(alias="usedBy")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .odp_setup_processing_region import OdpSetupProcessingRegion


class OdpSetup(UniversalBaseModel):
    allow_ach: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="allowAch"),
        pydantic.Field(alias="allowAch", description="Enables or disables ACH payout functionality"),
    ] = None
    allow_checks: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="allowChecks"),
        pydantic.Field(alias="allowChecks", description="Enables or disables check printing payout functionality"),
    ] = None
    allow_v_card: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="allowVCard"),
        pydantic.Field(alias="allowVCard", description="Enables or disables vCard payout functionality"),
    ] = None
    processing_region: typing.Optional[OdpSetupProcessingRegion] = pydantic.Field(default=None)
    """
    Region where payment processing occurs
    """

    processor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Payment processor identifier
    """

    issuer_network_settings_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="issuerNetworkSettingsId"),
        pydantic.Field(
            alias="issuerNetworkSettingsId", description="Reference ID for the program enabled for ODP issuance"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .billing_address_addtl_nullable import BillingAddressAddtlNullable
from .billing_address_nullable import BillingAddressNullable
from .billing_city_nullable import BillingCityNullable
from .billing_country_nullable import BillingCountryNullable
from .billing_state_nullable import BillingStateNullable
from .billing_zip import BillingZip
from .customer_id import CustomerId
from .customer_number_nullable import CustomerNumberNullable
from .email import Email
from .identifierfields import Identifierfields
from .phone_number import PhoneNumber
from .shippingaddress import Shippingaddress
from .shippingaddressadditional import Shippingaddressadditional
from .shippingcity import Shippingcity
from .shippingcountry import Shippingcountry
from .shippingstate import Shippingstate
from .shippingzip import Shippingzip


class PayorDataRequest(UniversalBaseModel):
    """
    Customer information. May be required, depending on the paypoint's settings. Required for subscriptions.
    """

    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="additionalData"), pydantic.Field(alias="additionalData")
    ] = None
    billing_address_1: typing_extensions.Annotated[
        typing.Optional[BillingAddressNullable],
        FieldMetadata(alias="billingAddress1"),
        pydantic.Field(alias="billingAddress1"),
    ] = None
    billing_address_2: typing_extensions.Annotated[
        typing.Optional[BillingAddressAddtlNullable],
        FieldMetadata(alias="billingAddress2"),
        pydantic.Field(alias="billingAddress2"),
    ] = None
    billing_city: typing_extensions.Annotated[
        typing.Optional[BillingCityNullable], FieldMetadata(alias="billingCity"), pydantic.Field(alias="billingCity")
    ] = None
    billing_country: typing_extensions.Annotated[
        typing.Optional[BillingCountryNullable],
        FieldMetadata(alias="billingCountry"),
        pydantic.Field(alias="billingCountry"),
    ] = None
    billing_email: typing_extensions.Annotated[
        typing.Optional[Email], FieldMetadata(alias="billingEmail"), pydantic.Field(alias="billingEmail")
    ] = None
    billing_phone: typing_extensions.Annotated[
        typing.Optional[PhoneNumber], FieldMetadata(alias="billingPhone"), pydantic.Field(alias="billingPhone")
    ] = None
    billing_state: typing_extensions.Annotated[
        typing.Optional[BillingStateNullable], FieldMetadata(alias="billingState"), pydantic.Field(alias="billingState")
    ] = None
    billing_zip: typing_extensions.Annotated[
        typing.Optional[BillingZip],
        FieldMetadata(alias="billingZip"),
        pydantic.Field(
            alias="billingZip",
            description='Customer\'s billing ZIP code. For Pay In functions, this field supports 5-digit and 9-digit ZIP codes and alphanumeric Canadian postal codes. For example: "37615-1234" or "37615".',
        ),
    ] = None
    company: typing.Optional[str] = pydantic.Field(default=None)
    """
    Customer's company name.
    """

    customer_id: typing_extensions.Annotated[
        typing.Optional[CustomerId], FieldMetadata(alias="customerId"), pydantic.Field(alias="customerId")
    ] = None
    customer_number: typing_extensions.Annotated[
        typing.Optional[CustomerNumberNullable],
        FieldMetadata(alias="customerNumber"),
        pydantic.Field(alias="customerNumber"),
    ] = None
    first_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="firstName"),
        pydantic.Field(alias="firstName", description="Customer/Payor first name."),
    ] = None
    identifier_fields: typing_extensions.Annotated[
        typing.Optional[Identifierfields],
        FieldMetadata(alias="identifierFields"),
        pydantic.Field(alias="identifierFields"),
    ] = None
    last_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="lastName"),
        pydantic.Field(alias="lastName", description="Customer/Payor last name."),
    ] = None
    shipping_address_1: typing_extensions.Annotated[
        typing.Optional[Shippingaddress],
        FieldMetadata(alias="shippingAddress1"),
        pydantic.Field(alias="shippingAddress1"),
    ] = None
    shipping_address_2: typing_extensions.Annotated[
        typing.Optional[Shippingaddressadditional],
        FieldMetadata(alias="shippingAddress2"),
        pydantic.Field(alias="shippingAddress2"),
    ] = None
    shipping_city: typing_extensions.Annotated[
        typing.Optional[Shippingcity], FieldMetadata(alias="shippingCity"), pydantic.Field(alias="shippingCity")
    ] = None
    shipping_country: typing_extensions.Annotated[
        typing.Optional[Shippingcountry],
        FieldMetadata(alias="shippingCountry"),
        pydantic.Field(alias="shippingCountry"),
    ] = None
    shipping_state: typing_extensions.Annotated[
        typing.Optional[Shippingstate], FieldMetadata(alias="shippingState"), pydantic.Field(alias="shippingState")
    ] = None
    shipping_zip: typing_extensions.Annotated[
        typing.Optional[Shippingzip], FieldMetadata(alias="shippingZip"), pydantic.Field(alias="shippingZip")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class TransferOutDetailBillAttachment(UniversalBaseModel):
    """
    Attachment for a bill.
    """

    ftype: typing.Optional[str] = pydantic.Field(default=None)
    """
    File type.
    """

    filename: typing.Optional[str] = pydantic.Field(default=None)
    """
    File name.
    """

    file_descriptor: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="fileDescriptor"),
        pydantic.Field(alias="fileDescriptor", description="File descriptor."),
    ] = None
    furl: typing.Optional[str] = pydantic.Field(default=None)
    """
    File URL.
    """

    f_content: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="fContent"),
        pydantic.Field(alias="fContent", description="File content."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .key_value_duo import KeyValueDuo
from .notification_content_event_type import NotificationContentEventType
from .notification_content_file_format import NotificationContentFileFormat
from .notification_content_report_name import NotificationContentReportName
from .timezone import Timezone


class NotificationContent(UniversalBaseModel):
    event_type: typing_extensions.Annotated[
        typing.Optional[NotificationContentEventType],
        FieldMetadata(alias="eventType"),
        pydantic.Field(alias="eventType", description="The notification's event name."),
    ] = None
    file_format: typing_extensions.Annotated[
        typing.Optional[NotificationContentFileFormat],
        FieldMetadata(alias="fileFormat"),
        pydantic.Field(
            alias="fileFormat",
            description="Indicate the format of report file to be generated by the engine.\nUsed for `method` = *report-email* and *report-web*.",
        ),
    ] = None
    internal_data: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueDuo]],
        FieldMetadata(alias="internalData"),
        pydantic.Field(
            alias="internalData",
            description="Array of pairs key:value to insert in request body to target in **method** = *web* or *report-web*.",
        ),
    ] = None
    report_name: typing_extensions.Annotated[
        typing.Optional[NotificationContentReportName],
        FieldMetadata(alias="reportName"),
        pydantic.Field(
            alias="reportName",
            description="The kind report to generate. For [automated reports](/developers/developer-guides/notifications-and-webhooks-overview#automated-reports) only.",
        ),
    ] = None
    time_zone: typing_extensions.Annotated[
        typing.Optional[Timezone], FieldMetadata(alias="timeZone"), pydantic.Field(alias="timeZone")
    ] = None
    transaction_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="transactionId"),
        pydantic.Field(
            alias="transactionId", description="Used internally to reference the entity or object generating the event."
        ),
    ] = None
    web_header_parameters: typing_extensions.Annotated[
        typing.Optional[typing.List[KeyValueDuo]],
        FieldMetadata(alias="webHeaderParameters"),
        pydantic.Field(
            alias="webHeaderParameters",
            description="Array of pairs key:value to insert in header of request to target in **method** = *web* or *report-web*.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ach_holder import AchHolder
from .ach_holder_type import AchHolderType
from .ach_sec_code import AchSecCode
from .achaccount import Achaccount
from .achaccounttype import Achaccounttype
from .achrouting import Achrouting
from .device import Device


class PayMethodAch(UniversalBaseModel):
    ach_account: typing_extensions.Annotated[
        Achaccount,
        FieldMetadata(alias="achAccount"),
        pydantic.Field(
            alias="achAccount", description="Bank account number. This field is **required** when method = 'ach'."
        ),
    ]
    ach_account_type: typing_extensions.Annotated[
        typing.Optional[Achaccounttype],
        FieldMetadata(alias="achAccountType"),
        pydantic.Field(
            alias="achAccountType", description="Bank account type. This field is **required** when method = 'ach'."
        ),
    ] = None
    ach_code: typing_extensions.Annotated[
        typing.Optional[AchSecCode], FieldMetadata(alias="achCode"), pydantic.Field(alias="achCode")
    ] = None
    ach_holder: typing_extensions.Annotated[
        AchHolder, FieldMetadata(alias="achHolder"), pydantic.Field(alias="achHolder")
    ]
    ach_holder_type: typing_extensions.Annotated[
        typing.Optional[AchHolderType], FieldMetadata(alias="achHolderType"), pydantic.Field(alias="achHolderType")
    ] = None
    ach_routing: typing_extensions.Annotated[
        Achrouting,
        FieldMetadata(alias="achRouting"),
        pydantic.Field(
            alias="achRouting",
            description="ABA/routing number of bank account. This field is **required** when method = 'ach'.",
        ),
    ]
    device: typing.Optional[Device] = None
    method: typing.Literal["ach"] = "ach"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class TierItem(UniversalBaseModel):
    amountx_auth: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="amountxAuth"), pydantic.Field(alias="amountxAuth")
    ] = None
    high_pay_range: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="highPayRange"), pydantic.Field(alias="highPayRange")
    ] = None
    low_pay_range: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="lowPayRange"), pydantic.Field(alias="lowPayRange")
    ] = None
    percentx_auth: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="percentxAuth"), pydantic.Field(alias="percentxAuth")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.additional_data_string import AdditionalDataString
from ...types.billing_address_addtl_nullable import BillingAddressAddtlNullable
from ...types.billing_address_nullable import BillingAddressNullable
from ...types.billing_city_nullable import BillingCityNullable
from ...types.billing_country_nullable import BillingCountryNullable
from ...types.billing_state_nullable import BillingStateNullable
from ...types.billing_zip import BillingZip
from ...types.customer_id import CustomerId
from ...types.customer_number_nullable import CustomerNumberNullable
from ...types.customer_status import CustomerStatus
from ...types.email import Email
from ...types.identifierfields import Identifierfields
from ...types.phone_number import PhoneNumber
from ...types.shippingaddress import Shippingaddress
from ...types.shippingaddressadditional import Shippingaddressadditional
from ...types.shippingcity import Shippingcity
from ...types.shippingcountry import Shippingcountry
from ...types.shippingstate import Shippingstate
from ...types.shippingzip import Shippingzip


class TransactionDetailCustomer(UniversalBaseModel):
    """
    Customer information associated with the transaction
    """

    identifiers: typing.Optional[Identifierfields] = None
    first_name: typing_extensions.Annotated[str, FieldMetadata(alias="firstName"), pydantic.Field(alias="firstName")]
    last_name: typing_extensions.Annotated[str, FieldMetadata(alias="lastName"), pydantic.Field(alias="lastName")]
    company_name: typing_extensions.Annotated[
        str, FieldMetadata(alias="companyName"), pydantic.Field(alias="companyName")
    ]
    billing_address_1: typing_extensions.Annotated[
        BillingAddressNullable, FieldMetadata(alias="billingAddress1"), pydantic.Field(alias="billingAddress1")
    ]
    billing_address_2: typing_extensions.Annotated[
        BillingAddressAddtlNullable, FieldMetadata(alias="billingAddress2"), pydantic.Field(alias="billingAddress2")
    ]
    billing_city: typing_extensions.Annotated[
        BillingCityNullable, FieldMetadata(alias="billingCity"), pydantic.Field(alias="billingCity")
    ]
    billing_state: typing_extensions.Annotated[
        BillingStateNullable, FieldMetadata(alias="billingState"), pydantic.Field(alias="billingState")
    ]
    billing_zip: typing_extensions.Annotated[
        BillingZip, FieldMetadata(alias="billingZip"), pydantic.Field(alias="billingZip")
    ]
    billing_country: typing_extensions.Annotated[
        BillingCountryNullable, FieldMetadata(alias="billingCountry"), pydantic.Field(alias="billingCountry")
    ]
    billing_phone: typing_extensions.Annotated[
        PhoneNumber, FieldMetadata(alias="billingPhone"), pydantic.Field(alias="billingPhone")
    ]
    billing_email: typing_extensions.Annotated[
        Email, FieldMetadata(alias="billingEmail"), pydantic.Field(alias="billingEmail")
    ]
    customer_number: typing_extensions.Annotated[
        CustomerNumberNullable, FieldMetadata(alias="customerNumber"), pydantic.Field(alias="customerNumber")
    ]
    shipping_address_1: typing_extensions.Annotated[
        Shippingaddress, FieldMetadata(alias="shippingAddress1"), pydantic.Field(alias="shippingAddress1")
    ]
    shipping_address_2: typing_extensions.Annotated[
        Shippingaddressadditional, FieldMetadata(alias="shippingAddress2"), pydantic.Field(alias="shippingAddress2")
    ]
    shipping_city: typing_extensions.Annotated[
        Shippingcity, FieldMetadata(alias="shippingCity"), pydantic.Field(alias="shippingCity")
    ]
    shipping_state: typing_extensions.Annotated[
        Shippingstate, FieldMetadata(alias="shippingState"), pydantic.Field(alias="shippingState")
    ]
    shipping_zip: typing_extensions.Annotated[
        Shippingzip, FieldMetadata(alias="shippingZip"), pydantic.Field(alias="shippingZip")
    ]
    shipping_country: typing_extensions.Annotated[
        Shippingcountry, FieldMetadata(alias="shippingCountry"), pydantic.Field(alias="shippingCountry")
    ]
    customer_id: typing_extensions.Annotated[
        CustomerId, FieldMetadata(alias="customerId"), pydantic.Field(alias="customerId")
    ]
    customer_status: typing_extensions.Annotated[
        CustomerStatus, FieldMetadata(alias="customerStatus"), pydantic.Field(alias="customerStatus")
    ]
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalDataString],
        FieldMetadata(alias="additionalData"),
        pydantic.Field(alias="additionalData"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .orgid import Orgid
from .template_code import TemplateCode
from .template_content import TemplateContent
from .template_name import TemplateName


class TemplateData(UniversalBaseModel):
    """
    Object containing the template's data.
    """

    org_id: typing_extensions.Annotated[
        typing.Optional[Orgid],
        FieldMetadata(alias="orgId"),
        pydantic.Field(alias="orgId", description="The ID of the organization the template belongs to. "),
    ] = None
    pricing_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="pricingId"), pydantic.Field(alias="pricingId")
    ] = None
    template_code: typing_extensions.Annotated[
        typing.Optional[TemplateCode], FieldMetadata(alias="templateCode"), pydantic.Field(alias="templateCode")
    ] = None
    template_content: typing_extensions.Annotated[
        typing.Optional[TemplateContent],
        FieldMetadata(alias="templateContent"),
        pydantic.Field(alias="templateContent"),
    ] = None
    template_description: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="templateDescription"),
        pydantic.Field(alias="templateDescription", description="A description for the template."),
    ] = None
    template_name: typing_extensions.Annotated[
        typing.Optional[TemplateName], FieldMetadata(alias="templateName"), pydantic.Field(alias="templateName")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VendorResponseBillingData(UniversalBaseModel):
    """
    Object containing vendor's bank information
    """

    id: typing.Optional[int] = None
    account_id: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="accountId"), pydantic.Field(alias="accountId")
    ] = None
    nickname: typing.Optional[str] = None
    bank_name: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="bankName"), pydantic.Field(alias="bankName")
    ] = None
    routing_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="routingAccount"), pydantic.Field(alias="routingAccount")
    ] = None
    account_number: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="accountNumber"), pydantic.Field(alias="accountNumber")
    ] = None
    type_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="typeAccount"), pydantic.Field(alias="typeAccount")
    ] = None
    bank_account_holder_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="bankAccountHolderName"),
        pydantic.Field(alias="bankAccountHolderName"),
    ] = None
    bank_account_holder_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="bankAccountHolderType"),
        pydantic.Field(alias="bankAccountHolderType"),
    ] = None
    bank_account_function: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="bankAccountFunction"), pydantic.Field(alias="bankAccountFunction")
    ] = None
    verified: typing.Optional[bool] = None
    status: typing.Optional[int] = None
    services: typing.Optional[typing.List[typing.Any]] = None
    default: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .avgmonthly import Avgmonthly
from .avgticketamt import Avgticketamt
from .baddress_1 import Baddress1
from .baddress_2 import Baddress2
from .bank_data import BankData
from .bcity import Bcity
from .bcountry import Bcountry
from .binperson import Binperson
from .binphone import Binphone
from .binweb import Binweb
from .boarding_id import BoardingId
from .bphone import Bphone
from .bstate import Bstate
from .bsummary import Bsummary
from .busstartdate import Busstartdate
from .bzip import Bzip
from .contacts_field import ContactsField
from .created_at import CreatedAt
from .dbaname import Dbaname
from .ein import Ein
from .external_paypoint_id import ExternalPaypointId
from .external_processor_information import ExternalProcessorInformation
from .highticketamt import Highticketamt
from .idpaypoint import Idpaypoint
from .last_modified import LastModified
from .legalname import Legalname
from .license import License
from .licensestate import Licensestate
from .maddress import Maddress
from .maddress_1 import Maddress1
from .mcity import Mcity
from .mcountry import Mcountry
from .mstate import Mstate
from .mzip import Mzip
from .org_parent_name import OrgParentName
from .orgid import Orgid
from .own_type import OwnType
from .ownership import Ownership
from .paypoint_entry_config import PaypointEntryConfig
from .paypoint_summary import PaypointSummary
from .paypointstatus import Paypointstatus
from .sales_code import SalesCode
from .services import Services
from .taxfillname import Taxfillname
from .template_id import TemplateId
from .timezone import Timezone
from .website import Website
from .whencharged import Whencharged
from .whendelivered import Whendelivered
from .whenprovided import Whenprovided
from .whenrefunded import Whenrefunded


class QueryEntrypointResponseRecordsItem(UniversalBaseModel):
    average_monthly_volume: typing_extensions.Annotated[
        typing.Optional[Avgmonthly],
        FieldMetadata(alias="AverageMonthlyVolume"),
        pydantic.Field(alias="AverageMonthlyVolume"),
    ] = None
    average_ticket_amount: typing_extensions.Annotated[
        typing.Optional[Avgticketamt],
        FieldMetadata(alias="AverageTicketAmount"),
        pydantic.Field(alias="AverageTicketAmount"),
    ] = None
    b_address_1: typing_extensions.Annotated[
        typing.Optional[Baddress1], FieldMetadata(alias="BAddress1"), pydantic.Field(alias="BAddress1")
    ] = None
    b_address_2: typing_extensions.Annotated[
        typing.Optional[Baddress2], FieldMetadata(alias="BAddress2"), pydantic.Field(alias="BAddress2")
    ] = None
    bank_data: typing_extensions.Annotated[
        typing.Optional[BankData], FieldMetadata(alias="BankData"), pydantic.Field(alias="BankData")
    ] = None
    b_city: typing_extensions.Annotated[
        typing.Optional[Bcity], FieldMetadata(alias="BCity"), pydantic.Field(alias="BCity")
    ] = None
    b_country: typing_extensions.Annotated[
        typing.Optional[Bcountry], FieldMetadata(alias="BCountry"), pydantic.Field(alias="BCountry")
    ] = None
    b_fax: typing_extensions.Annotated[
        typing.Optional[Bphone],
        FieldMetadata(alias="BFax"),
        pydantic.Field(alias="BFax", description="The business's fax number."),
    ] = None
    bin_person: typing_extensions.Annotated[
        typing.Optional[Binperson], FieldMetadata(alias="BinPerson"), pydantic.Field(alias="BinPerson")
    ] = None
    bin_phone: typing_extensions.Annotated[
        typing.Optional[Binphone], FieldMetadata(alias="BinPhone"), pydantic.Field(alias="BinPhone")
    ] = None
    bin_web: typing_extensions.Annotated[
        typing.Optional[Binweb], FieldMetadata(alias="BinWeb"), pydantic.Field(alias="BinWeb")
    ] = None
    boarding_id: typing_extensions.Annotated[
        typing.Optional[BoardingId], FieldMetadata(alias="BoardingId"), pydantic.Field(alias="BoardingId")
    ] = None
    b_phone: typing_extensions.Annotated[
        typing.Optional[Bphone], FieldMetadata(alias="BPhone"), pydantic.Field(alias="BPhone")
    ] = None
    b_startdate: typing_extensions.Annotated[
        typing.Optional[Busstartdate], FieldMetadata(alias="BStartdate"), pydantic.Field(alias="BStartdate")
    ] = None
    b_state: typing_extensions.Annotated[
        typing.Optional[Bstate], FieldMetadata(alias="BState"), pydantic.Field(alias="BState")
    ] = None
    b_summary: typing_extensions.Annotated[
        typing.Optional[Bsummary], FieldMetadata(alias="BSummary"), pydantic.Field(alias="BSummary")
    ] = None
    b_time_zone: typing_extensions.Annotated[
        typing.Optional[Timezone], FieldMetadata(alias="BTimeZone"), pydantic.Field(alias="BTimeZone")
    ] = None
    b_zip: typing_extensions.Annotated[
        typing.Optional[Bzip], FieldMetadata(alias="BZip"), pydantic.Field(alias="BZip")
    ] = None
    contact_data: typing_extensions.Annotated[
        typing.Optional[ContactsField], FieldMetadata(alias="ContactData"), pydantic.Field(alias="ContactData")
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt"), pydantic.Field(alias="CreatedAt")
    ] = None
    dba_name: typing_extensions.Annotated[
        typing.Optional[Dbaname], FieldMetadata(alias="DbaName"), pydantic.Field(alias="DbaName")
    ] = None
    documents_ref: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="DocumentsRef"), pydantic.Field(alias="DocumentsRef")
    ] = None
    ein: typing_extensions.Annotated[typing.Optional[Ein], FieldMetadata(alias="Ein"), pydantic.Field(alias="Ein")] = (
        None
    )
    entry_points: typing_extensions.Annotated[
        typing.Optional[typing.List[PaypointEntryConfig]],
        FieldMetadata(alias="EntryPoints"),
        pydantic.Field(alias="EntryPoints"),
    ] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId],
        FieldMetadata(alias="externalPaypointID"),
        pydantic.Field(alias="externalPaypointID"),
    ] = None
    external_processor_information: typing_extensions.Annotated[
        typing.Optional[ExternalProcessorInformation],
        FieldMetadata(alias="ExternalProcessorInformation"),
        pydantic.Field(alias="ExternalProcessorInformation"),
    ] = None
    high_ticket_amount: typing_extensions.Annotated[
        typing.Optional[Highticketamt],
        FieldMetadata(alias="HighTicketAmount"),
        pydantic.Field(alias="HighTicketAmount"),
    ] = None
    id_paypoint: typing_extensions.Annotated[
        typing.Optional[Idpaypoint], FieldMetadata(alias="IdPaypoint"), pydantic.Field(alias="IdPaypoint")
    ] = None
    last_modified: typing_extensions.Annotated[
        typing.Optional[LastModified], FieldMetadata(alias="LastModified"), pydantic.Field(alias="LastModified")
    ] = None
    legal_name: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="LegalName"), pydantic.Field(alias="LegalName")
    ] = None
    license: typing_extensions.Annotated[
        typing.Optional[License], FieldMetadata(alias="License"), pydantic.Field(alias="License")
    ] = None
    license_state: typing_extensions.Annotated[
        typing.Optional[Licensestate], FieldMetadata(alias="LicenseState"), pydantic.Field(alias="LicenseState")
    ] = None
    m_address_1: typing_extensions.Annotated[
        typing.Optional[Maddress], FieldMetadata(alias="MAddress1"), pydantic.Field(alias="MAddress1")
    ] = None
    m_address_2: typing_extensions.Annotated[
        typing.Optional[Maddress1], FieldMetadata(alias="MAddress2"), pydantic.Field(alias="MAddress2")
    ] = None
    mccid: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="Mccid"), pydantic.Field(alias="Mccid")
    ] = None
    m_city: typing_extensions.Annotated[
        typing.Optional[Mcity], FieldMetadata(alias="MCity"), pydantic.Field(alias="MCity")
    ] = None
    m_country: typing_extensions.Annotated[
        typing.Optional[Mcountry], FieldMetadata(alias="MCountry"), pydantic.Field(alias="MCountry")
    ] = None
    m_state: typing_extensions.Annotated[
        typing.Optional[Mstate], FieldMetadata(alias="MState"), pydantic.Field(alias="MState")
    ] = None
    m_zip: typing_extensions.Annotated[
        typing.Optional[Mzip], FieldMetadata(alias="MZip"), pydantic.Field(alias="MZip")
    ] = None
    org_id: typing_extensions.Annotated[
        typing.Optional[Orgid], FieldMetadata(alias="OrgId"), pydantic.Field(alias="OrgId")
    ] = None
    org_parent_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="OrgParentName"), pydantic.Field(alias="OrgParentName")
    ] = None
    owner_data: typing_extensions.Annotated[
        typing.Optional[Ownership], FieldMetadata(alias="OwnerData"), pydantic.Field(alias="OwnerData")
    ] = None
    own_type: typing_extensions.Annotated[
        typing.Optional[OwnType], FieldMetadata(alias="OwnType"), pydantic.Field(alias="OwnType")
    ] = None
    paypoint_status: typing_extensions.Annotated[
        typing.Optional[Paypointstatus], FieldMetadata(alias="PaypointStatus"), pydantic.Field(alias="PaypointStatus")
    ] = None
    sales_code: typing_extensions.Annotated[
        typing.Optional[SalesCode], FieldMetadata(alias="SalesCode"), pydantic.Field(alias="SalesCode")
    ] = None
    service_data: typing_extensions.Annotated[
        typing.Optional[Services], FieldMetadata(alias="ServiceData"), pydantic.Field(alias="ServiceData")
    ] = None
    summary: typing.Optional[PaypointSummary] = None
    taxfillname: typing_extensions.Annotated[
        typing.Optional[Taxfillname], FieldMetadata(alias="Taxfillname"), pydantic.Field(alias="Taxfillname")
    ] = None
    template_id: typing_extensions.Annotated[
        typing.Optional[TemplateId], FieldMetadata(alias="TemplateId"), pydantic.Field(alias="TemplateId")
    ] = None
    website_address: typing_extensions.Annotated[
        typing.Optional[Website],
        FieldMetadata(alias="WebsiteAddress"),
        pydantic.Field(alias="WebsiteAddress", description="Business website."),
    ] = None
    whencharged: typing_extensions.Annotated[
        typing.Optional[Whencharged], FieldMetadata(alias="Whencharged"), pydantic.Field(alias="Whencharged")
    ] = None
    whendelivered: typing_extensions.Annotated[
        typing.Optional[Whendelivered], FieldMetadata(alias="Whendelivered"), pydantic.Field(alias="Whendelivered")
    ] = None
    whenprovided: typing_extensions.Annotated[
        typing.Optional[Whenprovided], FieldMetadata(alias="Whenprovided"), pydantic.Field(alias="Whenprovided")
    ] = None
    whenrefund: typing_extensions.Annotated[
        typing.Optional[Whenrefunded], FieldMetadata(alias="Whenrefund"), pydantic.Field(alias="Whenrefund")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class NotificationLog(UniversalBaseModel):
    id: uuid.UUID = pydantic.Field()
    """
    The unique identifier for the notification.
    """

    org_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="orgId"),
        pydantic.Field(alias="orgId", description="The ID of the organization that the notification belongs to."),
    ] = None
    paypoint_id: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="paypointId"),
        pydantic.Field(alias="paypointId", description="The ID of the paypoint that the notification is related to."),
    ] = None
    notification_event: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="notificationEvent"),
        pydantic.Field(alias="notificationEvent", description="The event that triggered the notification."),
    ] = None
    target: typing.Optional[str] = pydantic.Field(default=None)
    """
    The target URL for the notification.
    """

    response_status: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="responseStatus"),
        pydantic.Field(alias="responseStatus", description="The HTTP response status of the notification."),
    ] = None
    success: bool = pydantic.Field()
    """
    Indicates whether the notification was successful.
    """

    job_data: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="jobData"),
        pydantic.Field(alias="jobData", description="Contains the body of the notification."),
    ] = None
    created_date: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdDate"),
        pydantic.Field(alias="createdDate", description="The date and time when the notification was created."),
    ]
    success_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="successDate"),
        pydantic.Field(
            alias="successDate", description="The date and time when the notification was successfully delivered."
        ),
    ] = None
    last_failed_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastFailedDate"),
        pydantic.Field(alias="lastFailedDate", description="The date and time when the notification last failed."),
    ] = None
    is_in_progress: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isInProgress"),
        pydantic.Field(
            alias="isInProgress", description="Indicates whether the notification is currently in progress."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .card_acceptance_element import CardAcceptanceElement
from .card_fee_section import CardFeeSection
from .card_flat_section import CardFlatSection
from .card_icp_section import CardIcpSection
from .card_pass_through_section import CardPassThroughSection
from .template_element import TemplateElement
from .visible import Visible


class CardService(UniversalBaseModel):
    batch_cutoff_time: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="batchCutoffTime"),
        pydantic.Field(
            alias="batchCutoffTime",
            description="Controls how to present the `batchCutoffTime` field on the application. If this field isn't sent, batch cut off time defaults to 5 ET.",
        ),
    ] = None
    card_acceptance: typing_extensions.Annotated[
        typing.Optional[CardAcceptanceElement],
        FieldMetadata(alias="cardAcceptance"),
        pydantic.Field(alias="cardAcceptance"),
    ] = None
    card_fees: typing_extensions.Annotated[
        typing.Optional[CardFeeSection], FieldMetadata(alias="cardFees"), pydantic.Field(alias="cardFees")
    ] = None
    card_flat: typing_extensions.Annotated[
        typing.Optional[CardFlatSection], FieldMetadata(alias="cardFlat"), pydantic.Field(alias="cardFlat")
    ] = None
    card_flat_amountx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardFlat_amountxAuth"),
        pydantic.Field(alias="cardFlat_amountxAuth"),
    ] = None
    card_flat_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardFlat_highPayRange"),
        pydantic.Field(alias="cardFlat_highPayRange"),
    ] = None
    card_flat_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardFlat_lowPayRange"),
        pydantic.Field(alias="cardFlat_lowPayRange"),
    ] = None
    card_flat_percentx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardFlat_percentxAuth"),
        pydantic.Field(alias="cardFlat_percentxAuth"),
    ] = None
    card_icp: typing_extensions.Annotated[
        typing.Optional[CardIcpSection], FieldMetadata(alias="cardICP"), pydantic.Field(alias="cardICP")
    ] = None
    card_icp_amountx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardICP_amountxAuth"),
        pydantic.Field(alias="cardICP_amountxAuth"),
    ] = None
    card_icp_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardICP_highPayRange"),
        pydantic.Field(alias="cardICP_highPayRange"),
    ] = None
    card_icp_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardICP_lowPayRange"),
        pydantic.Field(alias="cardICP_lowPayRange"),
    ] = None
    card_icp_percentx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardICP_percentxAuth"),
        pydantic.Field(alias="cardICP_percentxAuth"),
    ] = None
    card_pass_through: typing_extensions.Annotated[
        typing.Optional[CardPassThroughSection],
        FieldMetadata(alias="cardPassThrough"),
        pydantic.Field(alias="cardPassThrough"),
    ] = None
    card_pass_through_amount_recurring: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardPassThrough_amountRecurring"),
        pydantic.Field(alias="cardPassThrough_amountRecurring"),
    ] = None
    card_pass_through_amountx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardPassThrough_amountxAuth"),
        pydantic.Field(alias="cardPassThrough_amountxAuth"),
    ] = None
    card_pass_through_high_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardPassThrough_highPayRange"),
        pydantic.Field(alias="cardPassThrough_highPayRange"),
    ] = None
    card_pass_through_low_pay_range: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardPassThrough_lowPayRange"),
        pydantic.Field(alias="cardPassThrough_lowPayRange"),
    ] = None
    card_pass_through_percent_recurring: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardPassThrough_percentRecurring"),
        pydantic.Field(alias="cardPassThrough_percentRecurring"),
    ] = None
    card_pass_through_percentx_auth: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="cardPassThrough_percentxAuth"),
        pydantic.Field(alias="cardPassThrough_percentxAuth"),
    ] = None
    discount_frequency: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="discountFrequency"),
        pydantic.Field(alias="discountFrequency"),
    ] = None
    funding_rollup: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="fundingRollup"), pydantic.Field(alias="fundingRollup")
    ] = None
    gateway: typing.Optional[TemplateElement] = None
    pass_through_cost: typing_extensions.Annotated[
        typing.Optional[TemplateElement],
        FieldMetadata(alias="passThroughCost"),
        pydantic.Field(alias="passThroughCost"),
    ] = None
    pdf_template_id: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="pdfTemplateId"), pydantic.Field(alias="pdfTemplateId")
    ] = None
    pricing_plan: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="pricingPlan"), pydantic.Field(alias="pricingPlan")
    ] = None
    pricing_type: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="pricingType"), pydantic.Field(alias="pricingType")
    ] = None
    processor: typing.Optional[TemplateElement] = None
    provider: typing.Optional[TemplateElement] = None
    tier_name: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="tierName"), pydantic.Field(alias="tierName")
    ] = None
    visible: typing.Optional[Visible] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.authcode import Authcode
from ...types.avs_response_text import AvsResponseText
from ...types.cvv_response_text import CvvResponseText
from ...types.emv_auth_response_data import EmvAuthResponseData
from ...types.order_id import OrderId
from ...types.resulttext import Resulttext


class TransactionDetailResponseData(UniversalBaseModel):
    """
    Response data from payment processor
    """

    result_code: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resultCode"),
        pydantic.Field(
            alias="resultCode",
            description="Unified result code for the transaction. See [Pay In unified response codes](/guides/pay-in-unified-response-codes-reference) for more information.",
        ),
    ] = None
    result_code_text: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resultCodeText"),
        pydantic.Field(
            alias="resultCodeText",
            description="Description of the result code. See [Pay In unified response codes](/guides/pay-in-unified-response-codes-reference) for more information.",
        ),
    ] = None
    response: typing.Optional[str] = None
    responsetext: Resulttext
    authcode: typing.Optional[Authcode] = None
    transactionid: str
    avsresponse: typing.Optional[str] = None
    avsresponse_text: typing.Optional[AvsResponseText] = None
    cvvresponse: typing.Optional[str] = None
    cvvresponse_text: typing.Optional[CvvResponseText] = None
    orderid: typing.Optional[OrderId] = None
    type: typing.Optional[str] = None
    response_code: str
    response_code_text: str
    customer_vault_id: typing.Optional[str] = None
    emv_auth_response_data: typing.Optional[EmvAuthResponseData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class V2InternalServerError(UniversalBaseModel):
    """
    Internal server error response (HTTP 500) returned when an unexpected error occurs. Follows RFC 7807 Problem Details format.

    Examples
    --------
    from payabli.v_2_money_in_types import V2InternalServerError

    V2InternalServerError(
        title="Internal Server Error",
        status=500,
        detail="An unexpected error occurred while processing your request",
        instance="/api/v2/MoneyIn/getpaid",
    )
    """

    title: str = pydantic.Field()
    """
    Always "Internal Server Error" for 500 errors.
    """

    status: int = pydantic.Field()
    """
    HTTP status code, always 500 for internal errors.
    """

    detail: str = pydantic.Field()
    """
    Additional details about the internal error.
    """

    instance: str = pydantic.Field()
    """
    Request URL that caused the error.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ChargebackMessage(UniversalBaseModel):
    id: typing_extensions.Annotated[
        int, FieldMetadata(alias="Id"), pydantic.Field(alias="Id", description="Message identifier.")
    ]
    room_id: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="RoomId"),
        pydantic.Field(alias="RoomId", description="Room identifier for the message."),
    ]
    user_id: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="UserId"),
        pydantic.Field(alias="UserId", description="User identifier who sent the message."),
    ]
    user_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="UserName"),
        pydantic.Field(alias="UserName", description="Name of the user who sent the message."),
    ]
    content: typing_extensions.Annotated[
        str, FieldMetadata(alias="Content"), pydantic.Field(alias="Content", description="Content of the message.")
    ]
    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="CreatedAt"),
        pydantic.Field(alias="CreatedAt", description="Timestamp when the message was created."),
    ]
    message_type: typing_extensions.Annotated[
        int, FieldMetadata(alias="MessageType"), pydantic.Field(alias="MessageType", description="Type of message.")
    ]
    message_properties: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="MessageProperties"),
        pydantic.Field(alias="MessageProperties", description="Additional properties of the message."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PoiDevice(UniversalBaseModel):
    """
    Information about the point of interaction device (also known as a terminal or cloud device) used to process the transaction.
    """

    connected: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The device connection status.
    """

    date_de_registered: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="dateDeRegistered"),
        pydantic.Field(alias="dateDeRegistered", description="The date the device was unregistered."),
    ] = None
    date_registered: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="dateRegistered"),
        pydantic.Field(alias="dateRegistered", description="The date the device was registered."),
    ] = None
    device_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="deviceId"),
        pydantic.Field(alias="deviceId", description="The device identifier."),
    ] = None
    device_license: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="deviceLicense"),
        pydantic.Field(alias="deviceLicense", description="Device license. This is typically the same as `deviceId`."),
    ] = None
    device_nick_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="deviceNickName"),
        pydantic.Field(alias="deviceNickName", description="Device description provided during registration."),
    ] = None
    last_connected_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastConnectedDate"),
        pydantic.Field(alias="lastConnectedDate", description="Last connected date."),
    ] = None
    last_disconnected_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastDisconnectedDate"),
        pydantic.Field(alias="lastDisconnectedDate", description="Last disconnected date."),
    ] = None
    last_transaction_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="lastTransactionDate"),
        pydantic.Field(alias="lastTransactionDate", description="Last transaction date."),
    ] = None
    make: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device manufacturer.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device model.
    """

    registered: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The device registration status.
    """

    serial_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="serialNumber"),
        pydantic.Field(alias="serialNumber", description="The device serial number."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .page_identifier import PageIdentifier
from .payabli_api_response_error_400_response_data import PayabliApiResponseError400ResponseData


class PayabliApiResponseError400(UniversalBaseModel):
    is_success: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isSuccess"),
        pydantic.Field(
            alias="isSuccess",
            description="Boolean indicating whether the operation was successful. A `true` value indicates success. A `false` value indicates failure.",
        ),
    ] = None
    pageidentifier: typing.Optional[PageIdentifier] = None
    response_code: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="responseCode"),
        pydantic.Field(
            alias="responseCode",
            description="A code that indicates the operation's failure reason. See [API Response Codes](https://docs.payabli.com/api-reference/api-responses) for a full reference.",
        ),
    ] = None
    response_data: typing_extensions.Annotated[
        typing.Optional[PayabliApiResponseError400ResponseData],
        FieldMetadata(alias="responseData"),
        pydantic.Field(
            alias="responseData", description="Describes the reason for a failed operation and how to resolve it."
        ),
    ] = None
    response_text: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="responseText"),
        pydantic.Field(alias="responseText", description="Response text for operation: 'Success' or 'Declined'."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.authcode import Authcode
from ...types.avs_response import AvsResponse
from ...types.avs_response_text import AvsResponseText
from ...types.customer_vault_id import CustomerVaultId
from ...types.cvv_response import CvvResponse
from ...types.cvv_response_text import CvvResponseText
from ...types.emv_auth_response_data import EmvAuthResponseData
from ...types.order_id import OrderId
from ...types.result_code_text import ResultCodeText
from ...types.result_codev_2 import ResultCodev2
from ...types.resulttext import Resulttext


class V2TransactionDetailResponseData(UniversalBaseModel):
    """
    Response data from payment processor

    Examples
    --------
    from payabli.v_2_money_in_types import V2TransactionDetailResponseData

    V2TransactionDetailResponseData(
        result_code="A0000",
        result_code_text="Approved",
        responsetext="CAPTURED",
        authcode="AXS425",
        transactionid="TRN_Xo4dpKfmx3OxSc9svd2ccI6OOnyB2I",
        avsresponse="N",
        avsresponse_text="No Match, No address or ZIP match",
        cvvresponse="M",
        cvvresponse_text="CVV2/CVC2 match",
        response_code="100",
        response_code_text="Operation successful",
    )
    """

    result_code: typing_extensions.Annotated[
        ResultCodev2, FieldMetadata(alias="resultCode"), pydantic.Field(alias="resultCode")
    ]
    result_code_text: typing_extensions.Annotated[
        ResultCodeText, FieldMetadata(alias="resultCodeText"), pydantic.Field(alias="resultCodeText")
    ]
    response: typing.Optional[str] = None
    responsetext: Resulttext
    authcode: typing.Optional[Authcode] = None
    transactionid: str = pydantic.Field()
    """
    Unique identifier for the transaction assigned by the payment processor.
    """

    avsresponse: typing.Optional[AvsResponse] = None
    avsresponse_text: typing.Optional[AvsResponseText] = None
    cvvresponse: typing.Optional[CvvResponse] = None
    cvvresponse_text: typing.Optional[CvvResponseText] = None
    orderid: typing.Optional[OrderId] = None
    type: typing.Optional[str] = None
    response_code: str = pydantic.Field()
    """
    Processor-specific response code.
    """

    response_code_text: str = pydantic.Field()
    """
    Description of the response code.
    """

    customer_vault_id: typing.Optional[CustomerVaultId] = None
    emv_auth_response_data: typing.Optional[EmvAuthResponseData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.file_content_ftype import FileContentFtype
from ..types.payabli_api_response import PayabliApiResponse
from .types.payabli_api_response_ocr import PayabliApiResponseOcr
from .types.type_result import TypeResult

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawOcrClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def ocr_document_form(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseOcr]:
        """
        Use this endpoint to upload an image file for OCR processing. The accepted file formats include PDF, JPG, JPEG, PNG, and GIF. Specify the desired type of result (either 'bill' or 'invoice') in the path parameter `typeResult`. The response will contain the OCR processing results, including extracted data such as bill number, vendor information, bill items, and more.

        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentForm/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def ocr_document_json(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PayabliApiResponseOcr]:
        """
        Use this endpoint to submit a Base64-encoded image file for OCR processing. The accepted file formats include PDF, JPG, JPEG, PNG, and GIF. Specify the desired type of result (either 'bill' or 'invoice') in the path parameter `typeResult`. The response will contain the OCR processing results, including extracted data such as bill number, vendor information, bill items, and more.

        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentJson/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOcrClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def ocr_document_form(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseOcr]:
        """
        Use this endpoint to upload an image file for OCR processing. The accepted file formats include PDF, JPG, JPEG, PNG, and GIF. Specify the desired type of result (either 'bill' or 'invoice') in the path parameter `typeResult`. The response will contain the OCR processing results, including extracted data such as bill number, vendor information, bill items, and more.

        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentForm/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def ocr_document_json(
        self,
        type_result: TypeResult,
        *,
        ftype: typing.Optional[FileContentFtype] = OMIT,
        filename: typing.Optional[str] = OMIT,
        furl: typing.Optional[str] = OMIT,
        f_content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PayabliApiResponseOcr]:
        """
        Use this endpoint to submit a Base64-encoded image file for OCR processing. The accepted file formats include PDF, JPG, JPEG, PNG, and GIF. Specify the desired type of result (either 'bill' or 'invoice') in the path parameter `typeResult`. The response will contain the OCR processing results, including extracted data such as bill number, vendor information, bill items, and more.

        Parameters
        ----------
        type_result : TypeResult

        ftype : typing.Optional[FileContentFtype]

        filename : typing.Optional[str]
            The name of the file to be uploaded

        furl : typing.Optional[str]
            Optional URL link to the file

        f_content : typing.Optional[str]
            Base64-encoded file content

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseOcr]
            OCR document processing result
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Import/ocrDocumentJson/{jsonable_encoder(type_result)}",
            method="POST",
            json={
                "ftype": ftype,
                "filename": filename,
                "furl": furl,
                "fContent": f_content,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseOcr,
                    parse_obj_as(
                        type_=PayabliApiResponseOcr,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
